# Memory Benchmark

The `MemoryBenchmark.ino` was compiled with each `FEATURE_*` and the flash
memory and static RAM sizes were recorded. The `FEATURE_BASELINE` selection is
the baseline, and its memory usage  numbers are subtracted from the subsequent
`FEATURE_*` memory usage.

**NOTE**: This file was auto-generated using `make README.md`. DO NOT EDIT.

**Version**: AceTime v1.6

## How to Regenerate

To regenerate this README.md:

```
$ make clean_benchmarks
$ make benchmarks
$ make README.md
```

The `make benchmarks` target uses `collect.sh` script which calls `auniter.sh`
(https://github.com/bxparks/AUniter) to invoke the Arduino IDE programmatically.
It produces a `*.txt` file with the flash and ram usage information (e.g.
`nano.txt`). It now takes about 16 minutes to generate the `*.txt` files on my
quad-core Intel Core i7-3840QM CPU @ 2.80GHz laptop.

The `make README.md` command calls the `generated_readme.py` Python script which
generates this `README.md` file. The ASCII tables below are generated by the
`generate_table.awk` script, which takes each `*.txt` file and converts it to an
ASCII table.

## Library Size Changes

In v1.3, the `BasicZoneManager` and `ExtendedZoneManager` classes were unified
under a new parent interface `ZoneManager`. This seems to have caused the flash
size to increase by around 1200 bytes on the AVR processors (Nano, Pro Micro),
about 500 bytes on a SAMD, about 800 bytes on a ESP8266, 100 bytes on a ESP32,
and 1400 bytes on a Teensy 3.2. The 8-bit processors suffer the most
flash size increase proportional to their limited 32kB limit.

Adding the `ZoneManager` interface simplifies a lot of the complexity with
saving and restoring time zones using the `TimeZoneData` object, and I think it
is worth the extra cost of flash size. The mitigating factor is that
applications targetted towards 8-bit processors will normally have fixed number
of timezones at compile time, so they can avoid using a `ZoneManager`, and avoid
this penalty in flash size.

In v1.4.1+, we removed the `ZoneInfo::transitionBufSize` field from the
`ZoneInfo` struct, which saves 1 byte on 8-bit processors (none on 32-bit
processors due to 4-byte alignment). We save 266 bytes for `BasicZoneManager`
and 386 bytes for `ExtendedZoneManager` when all the zones are loaded into the
zone registry.

Also for v1.4.1+, incorporating zoneName compression causes flash/ram usage to
increase by ~250/120 bytes when using only 1-2 zones, but *decreases* flash
consumption by 1200-2400 bytes when all the zones are loaded into the
`ZoneManager`.

In v1.5+, changing just a single method, `ZoneProcessorCache::getType()`, from a
`virtual` to a non-virtual method saves 250-350 bytes of flash memory when using
a `BasicZoneManager` or an `ExtendedZoneManager` on an 8-bit AVR processor.
Unexpectedly, the flash memory consumption *increases* slightly (~0-50 bytes)
for some ARM processors and the ESP32. Since those processors have far more
flash memory, this seems like a good tradeoff.

Also in v1.5+, changing `BasicZoneProcessor` and `ExtendedZoneProcessor` to be
subclasses of the templatized `BasicZoneProcessorTemplate` and
`ExtendedZoneProcessorTemplate` classes causes reduction of flash consumption by
250-400 bytes for 32-bit processors. Don't know why. (Very little difference for
8-bit AVR). Adding a `BrokerFactory` layer of indirection (to support more
complex ZoneProcessors and Brokers) causes flash memory to go up by 100-200
bytes.

In v1.6, support for `LinkRegistry` was added to `BasicZoneManager` and
`ExtendedZoneManager`. This increases the flash memory usage by 150-500 bytes
when using one of these classes due to the code required by `LinkRegistrar`.
This extra cost is incurred even if the `LinkRegistry` is set to 0 elements.
Each `LinkEntry` consumes 8 bytes (2 x `uint32_t`). So a `zonedb::kLinkRegistry`
with 183 elements uses 1464 extra bytes of flash; a `zonedbx::kLinkRegistry`
with 207 elements uses 1656 extra bytes.

## Arduino Nano

* 16MHz ATmega328P
* Arduino IDE 1.8.13
* Arduino AVR Boards 1.8.3

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |    448/   10 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |   1648/  123 |  1200/  113 |
| ZonedDateTime                          |   2440/  123 |  1992/  113 |
| Manual ZoneManager                     |   2684/  123 |  2236/  113 |
| Basic TimeZone (1 zone)                |   7346/  309 |  6898/  299 |
| Basic TimeZone (2 zones)               |   7850/  313 |  7402/  303 |
| BasicZoneManager (1 zone)              |   9072/  329 |  8624/  319 |
| BasicZoneManager (zones)               |  22038/  705 | 21590/  695 |
| BasicZoneManager (zones+thin links)    |  23574/  705 | 23126/  695 |
| BasicZoneManager (zones+fat links)     |  26130/  705 | 25682/  695 |
| Extended TimeZone (1 zone)             |  10252/  343 |  9804/  333 |
| Extended TimeZone (2 zones)            |  10894/  347 | 10446/  337 |
| ExtendedZoneManager (1 zone)           |  11946/  363 | 11498/  353 |
| ExtendedZoneManager (zones)            |  34002/  847 | 33554/  837 |
| ExtendedZoneManager (zones+thin links) |  35726/  847 | 35278/  837 |
| ExtendedZoneManager (zones+fat links)  |  38646/  847 | 38198/  837 |
|----------------------------------------+--------------+-------------|
| SystemClock                            |   5662/  282 |  5214/  272 |
| SystemClock+Basic TimeZone             |  10410/  456 |  9962/  446 |
| SystemClock+Extended TimeZone          |  13354/  490 | 12906/  480 |
+---------------------------------------------------------------------+

```

## Sparkfun Pro Micro

* 16 MHz ATmega32U4
* Arduino IDE 1.8.13
* SparkFun AVR Boards 1.1.13

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |   3464/  150 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |   4772/  263 |  1308/  113 |
| ZonedDateTime                          |   5564/  263 |  2100/  113 |
| Manual ZoneManager                     |   5808/  263 |  2344/  113 |
| Basic TimeZone (1 zone)                |  10446/  447 |  6982/  297 |
| Basic TimeZone (2 zones)               |  10952/  453 |  7488/  303 |
| BasicZoneManager (1 zone)              |  12172/  467 |  8708/  317 |
| BasicZoneManager (zones)               |  25140/  845 | 21676/  695 |
| BasicZoneManager (zones+thin links)    |  26676/  845 | 23212/  695 |
| BasicZoneManager (zones+fat links)     |  29232/  845 | 25768/  695 |
| Extended TimeZone (1 zone)             |  13352/  481 |  9888/  331 |
| Extended TimeZone (2 zones)            |  13996/  487 | 10532/  337 |
| ExtendedZoneManager (1 zone)           |  15046/  501 | 11582/  351 |
| ExtendedZoneManager (zones)            |  37102/  985 | 33638/  835 |
| ExtendedZoneManager (zones+thin links) |  38826/  985 | 35362/  835 |
| ExtendedZoneManager (zones+fat links)  |  41746/  985 | 38282/  835 |
|----------------------------------------+--------------+-------------|
| SystemClock                            |   8646/  422 |  5182/  272 |
| SystemClock+Basic TimeZone             |  13392/  594 |  9928/  444 |
| SystemClock+Extended TimeZone          |  16336/  628 | 12872/  478 |
+---------------------------------------------------------------------+

```

## SAMD21 M0 Mini

* 48 MHz ARM Cortex-M0+
* Arduino IDE 1.8.13
* Sparkfun SAMD Boards 1.8.1

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |  10064/    0 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  10920/    0 |   856/    0 |
| ZonedDateTime                          |  11160/    0 |  1096/    0 |
| Manual ZoneManager                     |  11176/    0 |  1112/    0 |
| Basic TimeZone (1 zone)                |  15372/    0 |  5308/    0 |
| Basic TimeZone (2 zones)               |  15628/    0 |  5564/    0 |
| BasicZoneManager (1 zone)              |  16284/    0 |  6220/    0 |
| BasicZoneManager (zones)               |  33828/    0 | 23764/    0 |
| BasicZoneManager (zones+thin links)    |  35332/    0 | 25268/    0 |
| BasicZoneManager (zones+fat links)     |  40068/    0 | 30004/    0 |
| Extended TimeZone (1 zone)             |  17124/    0 |  7060/    0 |
| Extended TimeZone (2 zones)            |  17444/    0 |  7380/    0 |
| ExtendedZoneManager (1 zone)           |  18044/    0 |  7980/    0 |
| ExtendedZoneManager (zones)            |  48116/    0 | 38052/    0 |
| ExtendedZoneManager (zones+thin links) |  49812/    0 | 39748/    0 |
| ExtendedZoneManager (zones+fat links)  |  55204/    0 | 45140/    0 |
|----------------------------------------+--------------+-------------|
| SystemClock                            |  13264/    0 |  3200/    0 |
| SystemClock+Basic TimeZone             |  16876/    0 |  6812/    0 |
| SystemClock+Extended TimeZone          |  18764/    0 |  8700/    0 |
+---------------------------------------------------------------------+

```

(SAMD compiler does not produce RAM usage numbers.)

## STM32 Blue Pill

* STM32F103C8, 72 MHz ARM Cortex-M3
* Arduino IDE 1.8.13
* STM32duino 1.9.0

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |  19136/ 3788 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  23128/ 3988 |  3992/  200 |
| ZonedDateTime                          |  23280/ 3988 |  4144/  200 |
| Manual ZoneManager                     |  23288/ 3988 |  4152/  200 |
| Basic TimeZone (1 zone)                |  27180/ 3988 |  8044/  200 |
| Basic TimeZone (2 zones)               |  27452/ 3988 |  8316/  200 |
| BasicZoneManager (1 zone)              |  28040/ 3988 |  8904/  200 |
| BasicZoneManager (zones)               |  45316/ 3988 | 26180/  200 |
| BasicZoneManager (zones+thin links)    |  46812/ 3988 | 27676/  200 |
| BasicZoneManager (zones+fat links)     |  51416/ 3988 | 32280/  200 |
| Extended TimeZone (1 zone)             |  28692/ 3988 |  9556/  200 |
| Extended TimeZone (2 zones)            |  29008/ 3988 |  9872/  200 |
| ExtendedZoneManager (1 zone)           |  29628/ 3988 | 10492/  200 |
| ExtendedZoneManager (zones)            |  59284/ 3988 | 40148/  200 |
| ExtendedZoneManager (zones+thin links) |  60944/ 3988 | 41808/  200 |
| ExtendedZoneManager (zones+fat links)  |     -1/   -1 |    -1/   -1 |
|----------------------------------------+--------------+-------------|
| SystemClock                            |  26004/ 3988 |  6868/  200 |
| SystemClock+Basic TimeZone             |  29628/ 3988 | 10492/  200 |
| SystemClock+Extended TimeZone          |  31228/ 3988 | 12092/  200 |
+---------------------------------------------------------------------+

```

An entry of `-1` indicates that the memory usage exceeded the maximum of the
microcontroller and the compiler did not generate the desired information.

## ESP8266

* NodeMCU 1.0, 80MHz ESP8266
* Arduino IDE 1.8.13
* ESP8266 Boards 2.7.4

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               | 256700/26776 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          | 258820/27264 |  2120/  488 |
| ZonedDateTime                          | 259012/27264 |  2312/  488 |
| Manual ZoneManager                     | 259076/27264 |  2376/  488 |
| Basic TimeZone (1 zone)                | 264908/27824 |  8208/ 1048 |
| Basic TimeZone (2 zones)               | 265260/27824 |  8560/ 1048 |
| BasicZoneManager (1 zone)              | 266140/27824 |  9440/ 1048 |
| BasicZoneManager (zones)               | 283788/27824 | 27088/ 1048 |
| BasicZoneManager (zones+thin links)    | 285308/27824 | 28608/ 1048 |
| BasicZoneManager (zones+fat links)     | 289884/27824 | 33184/ 1048 |
| Extended TimeZone (1 zone)             | 267020/27968 | 10320/ 1192 |
| Extended TimeZone (2 zones)            | 267356/27968 | 10656/ 1192 |
| ExtendedZoneManager (1 zone)           | 268252/27968 | 11552/ 1192 |
| ExtendedZoneManager (zones)            | 298480/27980 | 41780/ 1204 |
| ExtendedZoneManager (zones+thin links) | 300176/27980 | 43476/ 1204 |
| ExtendedZoneManager (zones+fat links)  | 305392/27980 | 48692/ 1204 |
|----------------------------------------+--------------+-------------|
| SystemClock                            | 262268/27276 |  5568/  500 |
| SystemClock+Basic TimeZone             | 267720/27824 | 11020/ 1048 |
| SystemClock+Extended TimeZone          | 269848/27968 | 13148/ 1192 |
+---------------------------------------------------------------------+

```

## ESP32

* ESP32-01 Dev Board, 240 MHz Tensilica LX6
* Arduino IDE 1.8.13
* ESP32 Boards 1.0.4

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               | 206435/14564 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          | 218409/16124 | 11974/ 1560 |
| ZonedDateTime                          | 218717/16124 | 12282/ 1560 |
| Manual ZoneManager                     | 218765/16124 | 12330/ 1560 |
| Basic TimeZone (1 zone)                | 223373/16124 | 16938/ 1560 |
| Basic TimeZone (2 zones)               | 223745/16124 | 17310/ 1560 |
| BasicZoneManager (1 zone)              | 224481/16124 | 18046/ 1560 |
| BasicZoneManager (zones)               | 242193/16124 | 35758/ 1560 |
| BasicZoneManager (zones+thin links)    | 243689/16124 | 37254/ 1560 |
| BasicZoneManager (zones+fat links)     | 248537/16124 | 42102/ 1560 |
| Extended TimeZone (1 zone)             | 225233/16124 | 18798/ 1560 |
| Extended TimeZone (2 zones)            | 225601/16124 | 19166/ 1560 |
| ExtendedZoneManager (1 zone)           | 226357/16124 | 19922/ 1560 |
| ExtendedZoneManager (zones)            | 256685/16124 | 50250/ 1560 |
| ExtendedZoneManager (zones+thin links) | 258369/16124 | 51934/ 1560 |
| ExtendedZoneManager (zones+fat links)  | 263893/16124 | 57458/ 1560 |
|----------------------------------------+--------------+-------------|
| SystemClock                            | 225997/16228 | 19562/ 1664 |
| SystemClock+Basic TimeZone             | 230225/16228 | 23790/ 1664 |
| SystemClock+Extended TimeZone          | 232121/16228 | 25686/ 1664 |
+---------------------------------------------------------------------+

```

RAM usage remains constant as more objects are created, which indicates that an
initial pool of a certain minimum size is created regardless of the actual RAM
usage by objects.

## Teensy 3.2

* 96 MHz ARM Cortex-M4
* Arduino IDE 1.8.13
* Teensyduino 1.53

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |   7624/ 3048 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  13268/ 4812 |  5644/ 1764 |
| ZonedDateTime                          |  13268/ 4812 |  5644/ 1764 |
| Manual ZoneManager                     |  13268/ 4812 |  5644/ 1764 |
| Basic TimeZone (1 zone)                |  22392/ 4812 | 14768/ 1764 |
| Basic TimeZone (2 zones)               |  23120/ 4812 | 15496/ 1764 |
| BasicZoneManager (1 zone)              |  24484/ 4812 | 16860/ 1764 |
| BasicZoneManager (zones)               |  42164/ 4812 | 34540/ 1764 |
| BasicZoneManager (zones+thin links)    |  43692/ 4812 | 36068/ 1764 |
| BasicZoneManager (zones+fat links)     |  48508/ 4812 | 40884/ 1764 |
| Extended TimeZone (1 zone)             |  23908/ 4812 | 16284/ 1764 |
| Extended TimeZone (2 zones)            |  24636/ 4812 | 17012/ 1764 |
| ExtendedZoneManager (1 zone)           |  26000/ 4812 | 18376/ 1764 |
| ExtendedZoneManager (zones)            |  56360/ 4812 | 48736/ 1764 |
| ExtendedZoneManager (zones+thin links) |  58016/ 4812 | 50392/ 1764 |
| ExtendedZoneManager (zones+fat links)  |  63568/ 4812 | 55944/ 1764 |
|----------------------------------------+--------------+-------------|
| SystemClock                            |  16192/ 4812 |  8568/ 1764 |
| SystemClock+Basic TimeZone             |  25500/ 4812 | 17876/ 1764 |
| SystemClock+Extended TimeZone          |  27016/ 4812 | 19392/ 1764 |
+---------------------------------------------------------------------+

```

